﻿using System;
using System.Reflection.PortableExecutable;
using System.Runtime.InteropServices;
using System.Text;
using DeviceClasses;

/*
 * BasicSpectrometerDLL (C# Windows)
 * -----------------
 * Copyright 2025 by Avenir Photonics GmbH & Co. KG
 * 
 * This is a very simple demo to show the essential steps for measuring a spectrum.
 * The spectrometer is controlled via USB using the ZioLinkLibrary.dll.
 *
 * If you prefer direct access to the device (without a DLL) please see the
 * BasicSpectrometer demo instead.
 * 
 */

namespace BasicSpectrometerDLLConsole
{
    internal class Program
    {
        static void Main(string[] args)
        {
            Console.WriteLine("BasicSpectrometer");
            Console.WriteLine("-----------------");

            try
            {
                //******************** Open USB connection to spectrometer ********************

                Console.WriteLine("Searching for spectrometer:");

                Spectrometer[] devices = ZioLinkSpectrometer.FindDevices();
               
                if (devices.Length == 0)
                {
                    Console.WriteLine("No device found.");
                    return;
                }

                Spectrometer spectr = devices[0];

                spectr.Open(); //open connection to the device

                //******************** Read device information ********************

                Console.WriteLine("Model name: " + spectr.DeviceName);
                Console.WriteLine("Serial number: " + spectr.SerialNumber);

                int pixelCount = spectr.PixelCount;
                Console.WriteLine("Number of pixels: " + pixelCount);

                //******************** Read wavelengths ********************

                float[] wavelengths = spectr.Wavelengths;
                Console.WriteLine("Wavelength range: " + wavelengths[0].ToString("0.00") + " to " + wavelengths[pixelCount - 1].ToString("0.00") + " nm");

                //******************** Set exposure parameters ********************

                Console.WriteLine("Setting exposure parameters to: 20 ms, 10x averaging, no auto exposure");
                spectr.ExposureTime = 0.02f; //in seconds
                spectr.Averaging = 10;
                (spectr as ZioLinkSpectrometer).AutoExposure = false;

                //******************** Start Exposure ********************

                Console.WriteLine("Starting exposure ...");
                spectr.StartExposure(); //optional parameter: number of spectra

                //******************** Wait for exposure to be finished ********************

                SpectrStatus status;
                int availableSpectra;
                do { status = spectr.GetStatus(out availableSpectra); }
                while (status == SpectrStatus.TakingSpectrum);
                Console.WriteLine("Exposure finished. " + availableSpectra + " spectrum available in device.");

                //******************** Read spectrum from device ********************

                Console.WriteLine("Reading spectrum from device:");
                SpectrumData spectrumData = spectr.GetSpectrumData();

                //******************** Display values ********************

                //Now we have the full spectrum in the spectrumData class. It also includes some metadata, for example:
                Console.WriteLine("Load level: " + (spectrumData.LoadLevel * 100).ToString("0.00") + " %");

                Console.WriteLine("First 10 spectrum values:");
                for (int p = 0; p < 10; p++)
                    Console.WriteLine(wavelengths[p].ToString("0.0") + " nm: " + spectrumData.Values[p]);

                Console.WriteLine("Done.");
            }
            catch (Exception ex)
            {
                Console.WriteLine("An error occurrd: " + ex.Message);
            }
        }
    }
}
