
/*
 * BasicSpectrometerDLL (C++ Windows)
 * -----------------
 * Copyright 2025 by Avenir Photonics GmbH & Co. KG
 *
 * This is a very simple demo in C++/CLI (.NET framework version 4.8)
 * to show the essential steps for measuring a spectrum.
 * The spectrometer is controlled via USB using the ZioLinkLibrary.dll.
 *
 */

#include <iostream>
#include <string>
#include <cmath>

using namespace DeviceClasses;

int main()
{
    System::Console::WriteLine("BasicSpectrometer");
    System::Console::WriteLine("-----------------");

    try {
        // Find spectrometer
        System::Console::WriteLine("Searching for spectrometer:");

        array<Spectrometer^>^ devices = ZioLinkSpectrometer::FindDevices();

        if (devices->Length == 0) {
            System::Console::WriteLine("No device found.");
            return 0;
        }

        Spectrometer^ spectr = devices[0];

        spectr->Open(); // Open connection to the device

        // Read device information
        System::Console::WriteLine("Model name: " + spectr->DeviceName);
        System::Console::WriteLine("Serial number: + spectr->SerialNumber");

        int pixelCount = spectr->PixelCount;
        System::Console::WriteLine("Number of pixels: " + pixelCount);

        // Read wavelengths
        array<float>^ wavelengths = spectr->Wavelengths;
        System::Console::WriteLine("Wavelength range: " + wavelengths[0] + " to " + wavelengths[pixelCount - 1] + " nm");

        // Set exposure parameters
        System::Console::WriteLine("Setting exposure parameters to: 20 ms, 10x averaging, no auto exposure");
        spectr->ExposureTime = 0.02f; // in seconds
        spectr->Averaging = 10;
        static_cast<ZioLinkSpectrometer^>(spectr)->AutoExposure = false;

        // Start Exposure
        System::Console::WriteLine("Starting exposure ...");
        spectr->StartExposure(); // optional parameter: number of spectra

        // Wait for exposure to be finished
        SpectrStatus status;
        int availableSpectra;
        do { status = spectr->GetStatus(availableSpectra); } while (status == SpectrStatus::TakingSpectrum);
        System::Console::WriteLine("Exposure finished. " + availableSpectra + " spectrum available in device.");

        // Read spectrum from device
        System::Console::WriteLine("Reading spectrum from device:");
        SpectrumData^ spectrumData = spectr->GetSpectrumData();

        // Display values
        System::Console::WriteLine("Load level: " + (spectrumData->LoadLevel * 100) + " %");

        System::Console::WriteLine("First 10 spectrum values:");
        for (int p = 0; p < 10; p++)
            System::Console::WriteLine(wavelengths[p] + " nm: " + spectrumData->Values[p]);

        System::Console::WriteLine("Done.");
    }
    catch (System::Exception^ ex) {
        System::Console::WriteLine("An error occurred: " + ex->Message);
    }

    return 0;
}