﻿using System;
using System.Text;
using System.IO.Ports;
using System.Collections.Generic;
using System.Runtime.InteropServices;

namespace DeviceClasses
{
    public class SerialPortInterface : CommunicationInterface
    {
        public static string[] FindInterfaces()
        {
            return SerialPort.GetPortNames();
        }

        public SerialPortInterface(string devicePath)
        {
            this.devicePath = devicePath;
            port = new SerialPort(this.devicePath);
            port.ErrorReceived += Port_ErrorReceived;

            BaudRate = 57600;
            port.DataBits = 8;
            port.Parity = Parity.None;
            port.StopBits = StopBits.One;
            port.Handshake = Handshake.None;

            Timeout = 500;

            port.Encoding = Encoding.ASCII;
        }

        public override void Open()
        {
            port.Open();
        }

        public override void Close()
        {
            port.Close();
        }

        public override bool IsOpen { get { return port.IsOpen; } }

        public override string DevicePath { get { return devicePath; } }
        private string devicePath;

        protected SerialPort port;

        public int Timeout
        {
            get { return port.ReadTimeout; }
            set
            {
                port.ReadTimeout = value;
                port.WriteTimeout = value;
            }
        }

        public override string NewLine //may be null or empty
        {
            get
            {
                return (newLineAvailable ? port.NewLine : "");
            }
            set
            {
                newLineAvailable = !string.IsNullOrEmpty(value);
                if (newLineAvailable) port.NewLine = value;
            }
        }
        private bool newLineAvailable = false;

        public int BaudRate
        {
            get { return port.BaudRate; }
            set { port.BaudRate = value; }
        }

        public static int[] StandardBaudRates { get { return new int[] { 300, 1200, 2400, 4800, 9600, 19200, 38400, 57600, 115200, 230400, 460800, 921600, 3000000 }; } }

        public override int Read(byte[] buffer, int count)
        {
            checkLastError();
            int offset = 0;
            int numBytesRead;
            while (count > 0 && (numBytesRead = port.Read(buffer, offset, count)) > 0)
            {
                offset += numBytesRead;
                count -= numBytesRead;
            }
            return offset;
        }

        public override void Write(byte[] buffer, int count)
        {
            checkLastError();
            port.Write(buffer, 0, count);
        }

        public override byte ReadByte() { return (byte)port.ReadByte(); }

        public override string ReadLine()
        {
            if (!newLineAvailable) throw new InvalidOperationException("ReadString requires a line terminator.");
            return port.ReadLine();
        }

        public override void WriteLine(string str)
        {
            port.WriteLine(str);
        }

        public virtual int BytesToRead { get { return port.BytesToRead; } }

        private SerialError lastError = 0;

        private void Port_ErrorReceived(object sender, SerialErrorReceivedEventArgs e)
        {
            lastError = e.EventType;
        }

        protected void checkLastError()
        {
            if (lastError == 0) return;

            if ((lastError | SerialError.Frame) != 0) throw new CommunicationException("Serial port framing error.");
            if ((lastError | SerialError.Overrun) != 0) throw new CommunicationException("Serial port character buffer overrrun.");
            if ((lastError | SerialError.RXOver) != 0) throw new CommunicationException("Serial port input buffer overflow.");
            if ((lastError | SerialError.RXParity) != 0) throw new CommunicationException("Serial port parity error.");
            if ((lastError | SerialError.TXFull) != 0) throw new CommunicationException("Serial port output buffer full.");
            throw new CommunicationException("Unknown serial port error.");
        }

        public static bool PortIsAvailable(string devicePath)
        {
            try
            {
                SerialPort  testport = new SerialPort(devicePath);
                testport.Open();
                testport.Close();
                return true;
            }
            catch
            {
                return false;
            }
        }

        public static Dictionary<string, string> GetPortDescriptions() //key: serial port name (without colon), value: description
        {
            Dictionary<string, string> dict = new Dictionary<string, string>();
            Guid GUID_DEVCLASS_PORTS = new Guid("{4d36e978-e325-11ce-bfc1-08002be10318}");
            IntPtr hDeviceInfo = SetupDiGetClassDevs(ref GUID_DEVCLASS_PORTS, IntPtr.Zero, IntPtr.Zero, DIGCF_PRESENT);
            if (hDeviceInfo != IntPtr.Zero)
            {
                SP_DEVINFO_DATA deviceInfoData = new SP_DEVINFO_DATA() { cbSize = (uint)Marshal.SizeOf(typeof(SP_DEVINFO_DATA)) };
                uint n = 0;
                while (SetupDiEnumDeviceInfo(hDeviceInfo, n++, ref deviceInfoData))
                {
                    StringBuilder sbDeviceName = new StringBuilder(260);
                    bool success = SetupDiGetDeviceRegistryProperty(hDeviceInfo, ref deviceInfoData, SPDRP_FRIENDLYNAME, 0, sbDeviceName, (uint)sbDeviceName.Capacity, IntPtr.Zero);
                    if (success)
                    {
                        string s = sbDeviceName.ToString();
                        int p1 = s.LastIndexOf('(');
                        int p2 = s.LastIndexOf(')');
                        if (p1 > 1 && p2 > p1) dict.Add(s.Substring(p1 + 1, p2 - p1 - 1), s.Substring(0, p1 - 1));
                    }
                }
                SetupDiDestroyDeviceInfoList(hDeviceInfo);
            }
            return dict;
        }

        public static string[] FindInterfaces(int Vid, int Pid)
        {
            string id = "USB\\VID_" + Vid.ToString("X4");
            if (Pid >= 0) id += "&PID_" + Pid.ToString("X4");
            List<string> ports = new List<string>();
            Guid GUID_DEVCLASS_PORTS = new Guid("{4d36e978-e325-11ce-bfc1-08002be10318}");
            IntPtr hDeviceInfo = SetupDiGetClassDevs(ref GUID_DEVCLASS_PORTS, IntPtr.Zero, IntPtr.Zero, DIGCF_PRESENT);
            if (hDeviceInfo != IntPtr.Zero)
            {
                SP_DEVINFO_DATA deviceInfoData = new SP_DEVINFO_DATA() { cbSize = (uint)Marshal.SizeOf(typeof(SP_DEVINFO_DATA)) };
                uint n = 0;
                StringBuilder sb = new StringBuilder(260);
                while (SetupDiEnumDeviceInfo(hDeviceInfo, n++, ref deviceInfoData))
                {
                    bool success = SetupDiGetDeviceRegistryProperty(hDeviceInfo, ref deviceInfoData, SPDRP_HARDWAREID, 0, sb, (uint)sb.Capacity, IntPtr.Zero);
                    if (success)
                    {
                        if (sb.ToString().StartsWith(id, StringComparison.InvariantCultureIgnoreCase))
                        {
                            success = SetupDiGetDeviceRegistryProperty(hDeviceInfo, ref deviceInfoData, SPDRP_FRIENDLYNAME, 0, sb, (uint)sb.Capacity, IntPtr.Zero);
                            if (success)
                            {
                                string s = sb.ToString();
                                int p1 = s.LastIndexOf('(');
                                int p2 = s.LastIndexOf(')');
                                if (p1 > 1 && p2 > p1) ports.Add(s.Substring(p1 + 1, p2 - p1 - 1));
                            }
                        }
                    }
                }
                SetupDiDestroyDeviceInfoList(hDeviceInfo);
            }
            return ports.ToArray();
        }

        [DllImport("Setupapi.dll", CharSet = CharSet.Unicode, SetLastError = true)]
        public static extern IntPtr SetupDiGetClassDevs(ref Guid ClassGuid, IntPtr Enumerator, IntPtr hWndParent, int Flags);

        [DllImport("setupapi.dll", CharSet = CharSet.Auto, SetLastError = true)]
        public static extern bool SetupDiEnumDeviceInfo(IntPtr DeviceInfoSet, uint MemberIndex, ref SP_DEVINFO_DATA DeviceInfoData);

        [DllImport("Setupapi.dll", CharSet = CharSet.Unicode, SetLastError = true)]
        public static extern bool SetupDiGetDeviceRegistryProperty(IntPtr DeviceInfoSet, ref SP_DEVINFO_DATA DeviceInfoData, uint Property,
            uint PropertyRegDataType, StringBuilder PropertyBuffer, uint PropertyBufferSize, IntPtr RequiredSize);


        [DllImport("setupapi.dll", CharSet = CharSet.Auto, SetLastError = true)]
        public static extern bool SetupDiDestroyDeviceInfoList(IntPtr DeviceInfoSet);

        [StructLayout(LayoutKind.Sequential)]
        public struct SP_DEVINFO_DATA
        {
            public uint cbSize;
            public Guid ClassGuid;
            public uint DevInst;
            public IntPtr Reserved;
        }

        public const int DIGCF_PRESENT = 0x00000002;

        public const int SPDRP_DEVICEDESC = 0x00000000;  // DeviceDesc (R/W), from SetupAPI.h
        public const int SPDRP_HARDWAREID = 0x00000001;  // HardwareID (R/W), from SetupAPI.h
        public const int SPDRP_FRIENDLYNAME = 0x0000000C; // FriendlyName (R/W), from SetupAPI.h
    }
}
